### Name: buffer
### Title: Chemical Potential Buffers
### Aliases: buffer mod.buffer
### Keywords: misc

### ** Examples

  ## Don't show: 
data(thermo)
## End Don't show
  ## list the buffers
  thermo$buffer
  # another way to do it, for a specific buffer
  print(mod.buffer("PPM"))
  
  ## buffer of one species
  # calculate the activity of CO2 in equilibrium with
  # (a buffer made of) acetic acid at a given activity
  basis("CHNOS")
  basis("CO2","AC")
  # what activity of acetic acid are we using?
  print(mod.buffer("AC"))
  # return the activity of CO2
  affinity(return.buffer=TRUE)
  # as a function of oxygen fugacity
  affinity(O2=c(-85,-70,4),return.buffer=TRUE)
  # as a function of logfO2 and temperature
  affinity(O2=c(-85,-70,4),T=c(25,100,4),return.buffer=TRUE)
  # change the activity of species in the buffer
  mod.buffer("AC",logact=-10)
  affinity(O2=c(-85,-70,4),T=c(25,100,4),return.buffer=TRUE)

  ## show values of logfO2 on an Eh-pH diagram
  ## After Garrels, 1960, Figure 6
  basis("CHNOSe")
  # here we will buffer the activity of the electron by O2
  mod.buffer("O2","O2","gas",999)
  basis("e-","O2")
  # start our plot, then loop over values of logfO2
  thermo.plot.new(xlim=c(0,14),ylim=c(-0.8,1.2),
    xlab="pH",ylab=axis.label("Eh"))
  # the upper and lower lines correspond to the upper
  # and lower stability limits of water
  logfO2 <- c(0,-20,-40,-60,-83.1)
  for(i in 1:5) {
    # update the logarithm of fugacity (logact) of O2 in the buffer
    mod.buffer("O2","O2","gas",logfO2[i])
    # get the values of the logarithm of activity of the electron
    t <- affinity(pH=c(0,14,15),return.buffer=TRUE)
    # convert values of pe (-logact of the electron) to Eh
    Eh <- convert(-as.numeric(t[[1]]),"Eh")
    lines(seq(0,14,length.out=15),Eh)
    # add some labels
    text(seq(0,14,length.out=15)[i*2+2],Eh[i*2+2],
      paste("logfO2=",logfO2[i],sep=""))
  }
  title(main=paste("Relation between logfO2(g), Eh and pH at\n",
    "25 degC and 1 bar. After Garrels, 1960"))
  
  ## buffer of two species
  # conditions for metastable equilibrium among 
  # CO2 and acetic acid. note their starting activities:
  print(mod.buffer("CO2-AC")) 
  basis("CHNOS")
  basis("O2","CO2-AC")
  affinity(return.buffer=TRUE)  # logfO2 = -75.94248
  basis("CO2",123)  # what the buffer reactions are balanced on
  affinity(return.buffer=TRUE)  # unchanged
  # consider more oxidizing conditions
  mod.buffer("CO2-AC",logact=c(0,-10))
  affinity(return.buffer=TRUE)
  
  ## buffers of three species
  ## Pyrite-Pyrrhotite-Magnetite (PPM)
  # specify basis species and initial activities
  basis(c("FeS2","H2S","O2","H2O"),c(0,-10,-50,0))
  # note that the affinity of formation of pyrite,
  # which corresponds to FeS2 in the basis, is zero
  species(c("pyrite","pyrrhotite","magnetite"))
  affinity(T=c(200,400,11),P=2000)$values
  # attach basis species H2S and O2 to the PPM buffer
  basis(c("FeS2","H2S","O2","H2O"),c(0,"PPM","PPM",0))
  # inspect values of H2S activity and O2 fugacity
  affinity(T=c(200,400,11),P=2000,return.buffer=TRUE)
  # now, the affinities of formation reactions of 
  # species in the buffer are all equal to zero
  print(a <- affinity(T=c(200,400,11),P=2000)$values)
  for(i in 1:length(a)) stopifnot(isTRUE(
    all.equal(as.numeric(a[[i]]),rep(0,length(a[[i]])))))

  ### this example can be run using longex("buffer")
  ## Not run: 
##D     ## log fH2 - temperature diagram for mineral buffers
##D     ## and for given activities of aqueous CH2O and HCN
##D     ## After Schulte and Shock, 1995, Fig. 6: 
##D     ## 300 bars, log fCO2=1, log fN2=0, log aH2O=0
##D     # the mineral buffers FeFeO, QFM, PPM and HM are already
##D     # included in the thermo$buffer table so let's plot them.
##D     basis.logacts <- c(999,1,0,0,999,999,999)
##D     basis(c("Fe","CO2","H2O","nitrogen","hydrogen",
##D       "H2S","SiO2"),basis.logacts)
##D     basis(c("CO2","N2"),"gas")
##D     # initialize the plot
##D     xlim <- c(0,350)
##D     thermo.plot.new(xlim=xlim,ylim=c(-4,4),
##D       xlab=axis.label("T"),ylab=axis.label("H2"))
##D     res <- 50
##D     Tseq <- seq(xlim[1],xlim[2],length.out=res) 
##D     # a function to plot the log fH2 of buffers and label the lines
##D     logfH2plot <- function(buffer,lty,where) {
##D       basis("H2",buffer)
##D       t <- as.numeric(affinity(T=c(xlim,res),P=300,return.buffer=TRUE)$H2)
##D       lines(Tseq,t,lty=lty)
##D       # "where" is the percent distance along the x-axis to plot the label
##D       wherethis <- seq(xlim[1],xlim[2],length.out=100)[where]
##D       if(length(grep("_",buffer)) > 0) tt <- 
##D         thermo$buffer$logact[thermo$buffer$name==buffer] else tt <- buffer
##D       text(wherethis,splinefun(Tseq,t)(wherethis),tt)
##D     }
##D     # plot log fH2 of each mineral buffer
##D     logfH2plot("FeFeO",1,16)
##D     logfH2plot("QFM",1,30)
##D     logfH2plot("PPM",1,80)
##D     logfH2plot("HM",1,40)
##D     anotherplotfunction <- function(mybuff,lty,logact,where) {
##D       for(i in 1:length(logact)) {
##D         # update the species activity
##D         mod.buffer(mybuff,logact=logact[i])
##D         logfH2plot(mybuff,lty,where[i])
##D       }
##D     }
##D     # add and plot new buffers (formaldehyde and HCN)
##D     mod.buffer("mybuffer_1","formaldehyde","aq")
##D     logact <- c(-6,-10,-15); where <- c(10,10,25)
##D     anotherplotfunction("mybuffer_1",3,logact,where)
##D     mod.buffer("mybuffer_2","HCN","aq")
##D     where <- c(20,73,50)
##D     anotherplotfunction("mybuffer_2",2,logact,where)
##D     # title
##D     title(main=paste("Mineral buffers (solid), HCN (dashed), formaldehyde",
##D       "(dotted)\n",describe(thermo$basis[c(2,4),],T=NULL,P=300),
##D       "After Schulte and Shock, 1995"),cex.main=0.9)
##D   
## End(Not run)



