### Name: diagram
### Title: Calculate and Plot Relative Abundances of Species
### Aliases: diagram abundance.new abundance.old strip
### Keywords: misc

### ** Examples

  ## Don't show: 
data(thermo)
## End Don't show
  ### 0-D property and logarithm of activity plots
  ## properties of amino acids
  basis("CHNOS")
  basis("H2S",-25)
  aa <- c("glutamic acid","methionine","isoleucine",
    "leucine","tyrosine")
  species(aa)
  # Gibbs energy of formation reactions per CO2
  a <- affinity(property="G")
  diagram(a,balance=1)
  # affinity of reactions per CO2
  a <- affinity()
  diagram(a,property="A",do.plot=FALSE)
  ## logarithms of activities of amino acids
  # balanced on amino acid backbone, using abundance.old
  d1 <- diagram(a,balance=1,do.plot=FALSE)
  # balanced on amino acid backbone, using abundance.new
  d2 <- diagram(a,balance=1,residue=TRUE,do.plot=FALSE)
  # the last two give the same results
  stopifnot(all.equal(as.numeric(d2$logact),
    as.numeric(d1$logact)))
  ## 1-D plots
  a <- affinity(O2=c(-80,-70))
  # affinities of formation reactions
  d <- diagram(a,property="A",balance=1,ylim=c(-80,30))
  
  ### 1-D property plot
  ## for hydrous cordierite = cordierite + H2O
  ## after Helgeson et al., 1978
  basis(c("cordierite,hydrous","Mg+2","SiO2","H2O","O2","H+"))
  species("cordierite")
  # water.SUPCRT92 can only get us up to 5000 bar
  # (7000 and 10000 bar shown in the original)
  P <- c(1,2,3,5)*1000
  col <- rainbow(length(P))
  for(i in 1:length(P)) {
    a <- affinity(property="logK",T=c(20,800),P=P[i])
    diagram(a,add=(i!=1),ylim=c(-4,2),legend.x=NULL,
      color=col[i],title="")
  }
  legend("topright",lty=1,col=col,legend=paste(P,"bar"))
  title(main=paste("hydrous cordierite = cordierite + H2O",
    "After Helgeson et al., 1978",sep="\n"),cex.main=0.9)
  # this calculation could also be done through the
  # subcrt function, without having to set up basis species

  ### 1-D logarithm of activity plots
  ### (speciation diagrams)

  ## Aqueous sulfur species (after Seewald, 1997)
  basis("CHNOS+")
  basis("pH",5)
  species(c("H2S","S2-2","S3-2","S2O3-2","S2O4-2","S3O6-2",
    "S5O6-2","S2O6-2","HSO3-","SO2","HSO4-"))
  diagram(affinity(O2=c(-50,-15),T=325,P=350),
    ylim=c(-30,0),logact=-2,legend.x="topleft")
  title(main=paste("Aqueous sulfur speciation, 325 degC, 350 bar\n",
    "After Seewald, 1997"))
  # try it with and without the logact argument (total activity of 
  # the balanced quantity, in this case H2S aka sulfur) 

  ## Degrees of formation of ionized forms of glycine
  ## as a function of pH
  ## After Fig. 1 of Aksu and Doyle, 2001 
  basis("CHNOS+")
  species(ispecies <- info(c("glycinium","glycine","glycinate")))
  t <- affinity(pH=c(0,14))
  diagram(t,alpha=TRUE,lwd=1)
  title(main=paste("Degrees of formation of aqueous glycine species\n",
    "after Aksu and Doyle, 2001"))

  ### the rest of the examples can be run using longex("diagram")
  ## Not run: 
##D     ## Degrees of formation of ATP species as a function of 
##D     ## temperature, after LaRowe and Helgeson, 2007
##D     # Mg+2 here is the immobile component
##D     # cf. LH07, where bulk composition of Mg+2 is specified
##D     basis(c("CO2","NH3","H2O","H3PO4","O2","H+","Mg+2"),
##D       c(999,999,999,999,999,-5,-4))
##D     species(c("HATP-3","H2ATP-2","MgATP-2","MgHATP-"))
##D     diagram(affinity(T=c(0,120,25)),alpha=TRUE)  
##D     title(main=paste("Degrees of formation of ATP species,\n",
##D       "pH=5, log(aMg+2)=-3. After LaRowe and Helgeson, 2007"),
##D       cex.main=0.9)
##D 
##D     ## speciation of phosphate as a function of ionic strength
##D     basis("CHNOPS+")
##D     T <- c(25,100)
##D     species(c("HPO4-2","H2PO4-"))
##D     diagram(affinity(IS=c(0,0.25),T=T[1]),ylim=c(-3.2,-2.8))
##D     title(main=paste("Aqueous phosphate speciation, pH=7",
##D       "25 and 100 deg C - black and red lines",sep="\n"))
##D     diagram(affinity(IS=c(0,0.25),T=T[2]),ylim=c(-3.2,-2.8),add=TRUE,color="red")  
##D     ## phosphate predominance f(IS,pH)
##D     diagram(affinity(pH=c(6.8,7.2),IS=c(0,0.25),T=T[1]))
##D     title(main=paste("Aqueous phosphate predominance, 25 deg C",
##D       "and 100 deg C (dotted overlay)",sep="\n"))
##D     diagram(affinity(pH=c(6.8,7.2),IS=c(0,0.25),T=T[2]),add=TRUE,dotted=2,
##D       names=NULL)
##D 
##D     ## using strip(): equilibrium abundances of 
##D     ## proteins from different mammals
##D     organisms <- c("BOVIN","CANFA","HUMAN","MOUSE","PIG")
##D     proteins <- c("AQP1","MYG","P53")
##D     basis("CHNOS+")
##D     species(rep(proteins,each=5),organisms)
##D     a <- affinity(O2=c(-85,-65,250))
##D     ispecies <- list(1:5,6:10,11:15)
##D     desc <- c("(Aquaporin-1)","(Myoglobin)",
##D       "(Cellular tumor antigen p53)")
##D     names(ispecies) <- paste(proteins,desc)
##D     col <- rainbow(5)
##D     strip(a,ispecies=ispecies,ymin=-0.7,col=col)
##D     legend("bottomright",legend=organisms,col=col,
##D       lty=1,lwd=4,bty="n")
##D     title(main=paste("Equilibrium abundances of proteins",
##D       "from different mammals",sep="\n"))
##D     
##D     ### predominance diagrams (equal activities of
##D     ### species as a function of two variables) 
##D 
##D     ## T-P stability diagram for a unary system (SiO2)
##D     ## after Ernst, 1976, Fig. 4.4
##D     # the system is SiO2 (one component) but
##D     # the basis species require all the elements:
##D     # note that basis(c("SiO2","O2")) would identify SiO2(aq) 
##D     # which is okay but brings in calculations of properties 
##D     # of  water which are relatively slow.
##D     basis(c("quartz","O2"))  # cr, gas
##D     # browse the species of interest
##D     info("SiO2 ")
##D     # we'll take the crystalline ones
##D     t <- info("SiO2","cr")
##D     species(t)
##D     # calculate chemical affinities
##D     # the do.phases argument is necessary for 
##D     # dealing with the phase transitions of minerals
##D     t <- affinity(T=c(0,2000),P=c(0,30000),do.phases=TRUE)
##D     diagram(t)
##D     title(main="Phases of SiO2\nafter Ernst, 1976")
##D 
##D     ## Distribution of copper on Eh-pH diagram
##D     ## after Fig. 15 of Pourbaix, 1949
##D     basis(c("Cu+2","Cl-","H2O","H+","e-"))
##D     basis("Cl-",-2)
##D     # two aqueous species, three solid ones
##D     # (our CuCl is aq but it was cr in P's Fig. 15)
##D     species(c("CuCl","Cu+2","copper","cuprite","tenorite"))
##D     # (which is equivalent to ...)
##D     # species(c("CuCl","Cu+2","Cu","Cu2O","CuO"),c("","","","","","cr"))
##D     t <- affinity(pH=c(0,7),Eh=c(-0.1,1))
##D     # this one corresponds to activity contours of 
##D     # aqueous species at 10^-3 (the default aq activity in CHNOSZ)
##D     diagram(t,color=NULL)
##D     # here we set activities to unity; aq-cr boundaries change
##D     species(c("CuCl","Cu+2"),c(0,0))
##D     t <- affinity(pH=c(0,7),Eh=c(-0.1,1))
##D     diagram(t,add=TRUE,names=NULL,col="blue",color=NULL)
##D     water.lines()
##D     title(main=paste("H2O-Cl-(Cu); activities of 10^-3 (black)\n",
##D       "or 0 (blue); after Pourbaix, 1949"))
##D 
##D     ## Eh-pH diagrams for copper-water-glycine
##D     ## After Fig. 2 of Aksu and Doyle, 2001
##D     ##  We need to add some species and change some Gibbs energies.
##D     # update rows of the database
##D     i <- info(c("Cu(Gly)+","glycinate","e-","H+"))
##D     n <- nrow(thermo$obigt <- rbind(thermo$obigt,thermo$obigt[rep(i[1],2),]))
##D     thermo$obigt$name[n-1] <- "Cu(Gly)2-"
##D     thermo$obigt$name[n] <- "HCu(Gly)+2"
##D     thermo$obigt$formula[n-1] <- makeup(makeup(c(i[1],i[2],i[3]),""),"")
##D     thermo$obigt$formula[n] <- makeup(makeup(c(i[1],i[4]),""),"")
##D     # In Fig 2b, total log activities of Cu (Cu_T) 
##D     # and glycine (L_T) are -4 and -1
##D     basis(c("Cu+2","H2O","H+","e-","glycine","CO2"),c(999,0,999,999,-1,999))
##D     # solid species
##D     species(c("copper","cuprite","tenorite"))
##D     # aqueous species
##D     species(c("glycinium","glycine","glycinate","Cu+","Cu+2","CuO2-2","HCuO2-",
##D       "Cu(Gly)+","Cu(Gly)2","Cu(Gly)2-","HCu(Gly)+2"),-4)
##D     ispecies <- species()$ispecies
##D     # update the Gibbs energies using A&D's Table 1 and Table II
##D     logK <- c(convert(convert(c(0,-146,-129.7,-384.061,-370.647,-314.833,
##D       49.98,65.49,-183.6,-258.5,-298.2)*1000,"cal"),"logK"),15.64,10.1,2.92) 
##D     # do it in order so later species take account of prev. species' values
##D     for(i in 1:length(logK)) {
##D       G <- convert(logK[i],"G")
##D       if(i==12) G <- G + thermo$obigt$G[ispecies[8]] + 
##D         2*thermo$obigt$G[ispecies[6]]
##D       if(i==13) G <- G + thermo$obigt$G[ispecies[7]] + 
##D         2*thermo$obigt$G[ispecies[6]]
##D       if(i==14) G <- G + thermo$obigt$G[ispecies[11]]
##D       thermo$obigt$G[ispecies[i]] <- G
##D     }  # done with changing Gibbs free energies!
##D     # we have to get some leftovers out of there or diagram() gets confused
##D     species(c("glycinium","glycine","glycinate"),delete=TRUE)
##D     # make a plot to see if it's working
##D     ispecies <- ispecies[-(1:6)]
##D     afun <- function(cu,gly) {
##D       # from above: our fifth basis species is glycine(-ate,-ium)
##D       basis(rownames(basis())[5],gly)
##D       t <- match(ispecies,species()$ispecies)
##D       species(t,cu)
##D       affinity(pH=c(0,16),Eh=c(-0.6,1.0))
##D     }
##D     diagram(afun(-4,-1))
##D     title(main=paste("Aqueous Copper + Glycine, 25 deg C, 1 bar",
##D       "After Aksu and Doyle, 2001 Fig. 2b",sep="\n"))
##D     # What's missing? Try glycinate not glycine in reactions at ph > ~9.8
##D     basis(c("Cu+2","H2O","H+","e-","glycinate","CO2"),
##D       c(999,0,999,999,-2,999))
##D     species(c("copper","cuprite","tenorite","Cu+","Cu+2","CuO2-2","HCuO2-",
##D       "Cu(Gly)+","Cu(Gly)2","Cu(Gly)2-","HCu(Gly)+2"))
##D     loga_Cu <- -4
##D     loga_Gly <- -1
##D     diagram(afun(loga_Cu,loga_Gly),color=NULL,col="blue",
##D       names=species()$name,col.names="blue",add=TRUE)
##D     water.lines()
##D     # the glycine ionization constants could be calculated using
##D     # subcrt, here they are taken from A&D Table II
##D     abline(v=c(2.35,9.778),lty=3)
##D     # now put glycinium (low pH) in the basis
##D     basis(c("Cu+2","H2O","H+","e-","glycinium","CO2"),c(999,0,999,999,-2,999))
##D     species(c("copper","cuprite","tenorite","Cu+","Cu+2","CuO2-2","HCuO2-",
##D       "Cu(Gly)+","Cu(Gly)2","Cu(Gly)2-","HCu(Gly)+2"))
##D     diagram(afun(loga_Cu,loga_Gly),color=NULL,col="green",
##D       names=NULL,col.names="green",add=TRUE)
##D     # let's forget our changes to 'thermo' so the example 
##D     # below that uses glycine will work as expected
##D     data(thermo)
##D 
##D     ## a pe-pH diagram for hydrated iron sulfides,
##D     ## goethite and pyrite, After Majzlan et al., 2006
##D     basis(c("Fe+2","SO4-2","H2O","H+","e-"),c(0,log10(3),log10(0.75),999,999))
##D     species(c("rhomboclase","ferricopiapite","hydronium jarosite",
##D       "goethite","melanterite","pyrite"))
##D     a <- affinity(pH=c(-1,4),pe=c(-5,23))
##D     diagram(a)
##D     water.lines(yaxis="pe")
##D     title(main=paste("Fe-S-O-H After Majzlan et al., 2006",
##D       describe(thermo$basis[2:3,],digits=2),sep="\n"))
##D 
##D     ## cysteine-cysteinate-cystine Eh-pH
##D     ## at 25 and 100 deg C
##D     basis("CHNOSe")
##D     species(c("cysteine","cysteinate","cystine"))
##D     a <- affinity(pH=c(5,10),Eh=c(-0.5,0))
##D     diagram(a,color=NULL)
##D     water.lines()
##D     a <- affinity(pH=c(5,10),Eh=c(-0.5,0),T=100)
##D     diagram(a,col="red",add=TRUE,names=NULL)
##D     water.lines(T=convert(100,"K"),col="red")
##D     title(main=paste("Cysteine Cysteinate Cystine",
##D       "25 and 100 deg C",sep="\n"))
##D     
##D     ## Soil Organic Matter CO2-H2O, O2-H2O (after Tardy et al., 1997)
##D     # NH3 is conserved, and H2O is on an axis of the 
##D     # diagrams, so their activities are nonsensical here.
##D     # formulas for aqueous species, names for phases ...
##D     basis(c("NH3","water","CO2","O2"),c(999,999,-2.5,-28))
##D     # switch to gaseous CO2 (aq is the default)
##D     basis("CO2","gas")
##D     # load the species of interest
##D     species(c("microflore","plantes","acide fulvique",
##D       "acide humique","humine"))
##D     # proceed with the diagrams
##D     diagram(affinity(H2O=c(-0.6,0.1),CO2=c(-3,-1)))
##D     title(main=paste("Relative stabilities of soil organic matter\n",
##D       "after Tardy et al., 1997"))
##D     # this would be the O2-H2O diagram
##D     # diagram(affinity(H2O=c(-1,0.5),O2=c(-28.5,-27.5)))
##D 
##D     ## Aqueous Aluminum Species F-/OH- (afer Tagirov and Schott, 2001)
##D     # in order to reproduce this calculation, we have to 
##D     # consider the properties of species given by these authors,
##D     # which are not the default ones in the database
##D     # (see last example of 'thermo' help page)
##D     thermo$opt$level <- 2
##D     # The coefficients on H+ and O2 in all the formation reactions
##D     # are zero, so the number of basis species here is three. Al+3 
##D     # becomes the conservant, and F- and OH- are being plotted ...
##D     # so their initial activities don't have to be set.
##D     basis(c("Al+3","F-","OH-","H+","O2"),rep(999,5))
##D     species(c("Al+3","Al(OH)4-","AlOH+2","Al(OH)2+","Al(OH)3",
##D       "AlF+2","AlF2+","AlF3","AlF4-","Al(OH)2F2-","Al(OH)2F","AlOHF2"))
##D     # Increase the x- and y- resolution from the default and calculate
##D     # and plot predominance limits. Names of charged basis species,
##D     # such as "H+", "e-" and the ones shown here, should be quoted
##D     # when given as arguments to affinity(). The OH- values shown here
##D     # correspond to pH=c(0,14) (at unit activity of water).
##D     a <- affinity("OH-"=c(-14,0),"F-"=c(-1,-8),T=200)
##D     diagram(a)
##D     title(main=paste("Aqueous aluminium species, T=200 C, P=Psat\n",
##D       "after Tagirov and Schott, 2001"))
##D     # We could do this to overlay boundaries for a different pressure
##D     #a.P <- affinity("OH-"=c(-14,0),"F-"=c(-1,-8),T=200,P=5000)
##D     #diagram(a.P,names=NULL,color=NULL,col="blue",add=TRUE)
##D     # restore data selection level to default
##D     thermo$opt$level <- 1
##D 
##D     ## Mineral equilibrium activity diagram
##D     ## (After Bowers et al., 1984, p. 246)
##D     # Consider the system denoted by BJH84 as 
##D     # HCl-H2O-CaO-CO2-MgO-(SiO2) at 300 deg C and 1000 bar
##D     # HCl, CO2, O2 have zero stoichiometric coeffs in the species
##D     # Ca+2, Mg+2 are going to be on the diagram
##D     # SiO2 will be conserved (the immobile component)
##D     # (try changing any of the 999's, the diagram will be unaffected)
##D     basis(c("HCl","H2O","Ca+2","CO2","Mg+2","SiO2","O2","H+"),
##D           c(999,0,999,999,999,999,999,-7))
##D     # we have to tell CHNOSZ which species to include; there are
##D     # others that could be in this system
##D     species(c("quartz","talc","forsterite","tremolite","diopside",
##D       "wollastonite","monticellite","merwinite"))
##D     # calculate the chemical affinities of formation reactions
##D     #t <- affinity("Mg+2"=c(-14,-8),"Ca+2"=c(-12,-4),T=300,P=1000)
##D     t <- affinity("Mg+2"=c(-12,-4),"Ca+2"=c(-8,0),T=300,P=1000)
##D     diagram(t)
##D     title(main=paste("HCl-H2O-CaO-CO2-MgO-(SiO2) at 300 deg C,\n",
##D       "1000 bar and pH=7. After Bowers et al., 1984"))
##D     # note: BJH84 use a different method for representing 
##D     # the axes of the diagrams, similar to (a_Ca+2)/(a_H+)^2,
##D     # so this is so far an approximate reproduction of their diagram.
##D 
##D     ## Fe-S-O at 200 deg C, After Helgeson, 1970
##D     basis(c("Fe","O2","S2"))
##D     species(c("iron","ferrous-oxide","magnetite",
##D       "hematite","pyrite","pyrrhotite"))
##D     a <- affinity(S2=c(-50,0),O2=c(-90,-10),T=200)
##D     diagram(a,color="heat")
##D     title(main=paste("Fe-S-O, 200 degrees C, 1 bar",
##D       "After Helgeson, 1970",sep="\n"))
##D 
##D     ## Nucleobase - Amino Acid Interaction Eh-H2O
##D     # for this example we try a unique basis definition
##D     basis(c("CO2","H2O","glutamine","e-","H+"),c(-3,0,-3,0,-7))
##D     species(c("uracil","cytosine","adenine","guanine",
##D       "phenylalanine","proline","lysine","glycine"),"aq")
##D     # this loaded four nucleobases and four related amino acids
##D     # (coded for by the homocodon triplets)
##D     # check out the predominance diagrams
##D     a.1 <- affinity(H2O=c(-5,0),Eh=c(-0.5,0))
##D     diagram(a.1,color=NULL)
##D     # overlay a different temperature
##D     a.2 <- affinity(H2O=c(-5,0),Eh=c(-0.5,0),T=100)
##D     diagram(a.2,col="red",add=TRUE,names=NULL)
##D     # start make a title for the plot
##D     tb <- thermo$basis   # includes activities of basis species
##D     # exclude those that are on the axes
##D     tb <- tb[!((rownames(tb) %in% c("e-","H2O"))),]
##D     title(main=paste("Nucleobases and amino acids,",
##D       "T=25 and 100 C at Psat\n",
##D       describe(tb,T=NULL,P=NULL)),cex.main=0.9)
##D 
##D     ## Temperature-Pressure: kayanite-sillimanite-andalusite
##D     # this is a system of a single component (Al2SiO5)
##D     # but the number of basis species is the number of elements
##D     basis(c("kyanite","quartz","oxygen"))
##D     species(c("kyanite","sillimanite","andalusite"))
##D     diagram(affinity(T=c(0,1000,200),P=c(500,5000,200)),color=NULL)
##D     title(main="Al2SiO5")
##D   
## End(Not run)



