### Name: subcrt
### Title: Properties of Species and Reactions
### Aliases: subcrt read.supcrt write.supcrt
### Keywords: misc

### ** Examples

  ## Don't show: 
data(thermo)
## End Don't show
  ## properties of species
  subcrt("water")
  # calculating at different temperatures
  subcrt("water",T=seq(0,100,10))
  # calculating at even increments      
  subcrt("water",T=seq(500,1000,length.out=10),
    P=seq(5000,10000,length.out=10))
  # calculating on a temperature-pressure grid
  subcrt("water",T=c(500,1000),P=c(5000,10000),grid="P")
  # to calculate only selected properties
  subcrt("water",property=c("G","E"))   
  # the properties of multiple species
  subcrt(c("glucose","ethanol","CO2"))

  ## input/output in different units
  nuts(c("K","J"))
  subcrt("water")
  subcrt("water",T=298.15)
  nuts(c("cal","C"))   # back to defaults

  ## properties of reactions
  subcrt(c("H2O","H+","k-feldspar","kaolinite","K+","SiO2"),
    c(-1,-2,-2,1,2,4))
  subcrt(c("glucose","ethanol","CO2"),c(-1,2,2))
  # to specify the states
  subcrt(c("glucose","ethanol","CO2"),c(-1,2,2),c("aq","aq","gas")) 
  # this warns about an unbalanced reaction
  subcrt(c("glucose","ethanol"),c(-1,3))
  
  ## auto balancing reactions
  # the basis species must first be defined
  basis(c("CO2","H2O","NH3","H2S","O2"))
  subcrt(c("glucose","ethanol"),c(-1,3))
  # a bug in CHNOSZ <0.9 caused the following
  # to initiate an infinite loop
  basis(c("H2O","H2S","O2","H+"))
  subcrt(c("HS-","SO4-2"),c(-1,1))
  # note the next one is a non-reaction
  # (products same as reactants)
  subcrt("H2O",1)
  # but this one auto-balances into a reaction
  # (water to steam)
  subcrt("H2O",1,"gas")
  # properties of a species and a formation 
  # reaction for that species
  subcrt("C2H5OH")    # species
  basis("CHNOS")
  subcrt("C2H5OH",1)  # reaction
  
  ## properties of mineral phases
  # properties of phase species
  subcrt(c("pyrrhotite","pyrrhotite"),state=c("cr1","cr2"))
  # properties of the stable phases
  subcrt("pyrrhotite")
  # phase transitions in a reaction
  subcrt(c("pyrite","pyrrhotite","H2O","H2S","O2"),c(-1,1,-1,1,0.5))

  ## these produce NAs and warnings
  # Psat is not defined above the critical point
  subcrt("alanine",T=seq(0,5000,by=1000))
  # above the T, P limits for the H2O equations of state
  subcrt("alanine",T=c(2250,2251),P=c(30000,30001),grid="T")

  ### these examples can be run using longex("subcrt")
  ## Not run: 
##D     ## heat capacity of Fe(cr)
##D     # compare calculated values of heat capacity with
##D     # values from Robie and Hemingway, 1995, (from which 
##D     # the parameters in the database were derived)
##D     nuts(c("J","K"))  # set the units
##D     # we set pressure here otherwise subcrt goes for 
##D     # Psat (saturation vapor pressure of H2O above 
##D     # 100 degrees C) which can not be calculated above 
##D     # the critical point of H2O (~647 K)
##D     t <- subcrt("Fe",T=seq(300,1800,20),P=1)
##D     plot(t$out[[1]]$T,t$out[[1]]$Cp,type="l",
##D       xlab=axis.label("T"),ylab=axis.label("Cp"))
##D     # add points from RH95
##D     RH95 <- thermo$expt$RH95
##D     points(RH95[,1],RH95[,2])
##D     title(main=paste("Heat capacity of Fe(cr)\n",
##D       "(points - Robie and Hemingway, 1995)"))
##D     # reset the units
##D     nuts(c("C","cal"))
##D     
##D     ## Skarn example after Johnson et al., 1992
##D     P <- seq(500,5000,500)
##D     # this is like the temperature specification used 
##D     # in the example by Johnson et al., 1992
##D     # T <- seq(0,1000,100)
##D     # we use this one to avoid warnings at 0 deg C, 5000 bar
##D     T <- c(2,seq(100,1000,100))
##D     subcrt(c("andradite","carbon dioxide","H2S","Cu+","quartz","calcite",
##D       "chalcopyrite","H+","H2O"),coeff=c(-1,-3,-4,-2,3,3,2,2,3),
##D       state=c("cr","g","aq","aq","cr","cr","cr","aq","liq"),
##D       P=P,T=T,grid="P")
##D     # the volumes are significantly different from SUPCRT92
##D 
##D     ## Standard Gibbs energy of reactions with HCN and 
##D     ## formaldehyde, after Schulte and Shock, 1995 Fig. 1
##D     rxn1 <- subcrt(c("formaldehyde","HCN","H2O","glycolic acid","NH3"),
##D       c(-1,-1,-2,1,1),P=300)
##D     rxn2 <- subcrt(c("formaldehyde","HCN","H2O","glycine"),
##D       c(-1,-1,-1,1),P=300)
##D     plot(x=rxn1$out$T,rxn1$out$G/1000,type="l",ylim=c(-40,-10),
##D       xlab=axis.label("T"),ylab=axis.label("DG0r","k"))
##D     lines(rxn1$out$T,rxn2$out$G/1000)
##D     # write the reactions on the plot
##D     text(150,-14,describe(rxn1$reaction,
##D       use.name=c(TRUE,FALSE,FALSE,TRUE,FALSE)))
##D     text(200,-35,describe(rxn2$reaction,
##D       use.name=c(TRUE,FALSE,FALSE,TRUE)))
##D     title(main=paste("Standard Gibbs energy of reactions",
##D       "after Schulte and Shock, 1995",sep="\n"))
##D 
##D     ## Calculation of chemical affinities
##D     # after LaRowe and Helgeson, 2007
##D     # Fig. 3 (a): reduction of nicotinamide adenine 
##D     # dinucleotide (NAD) coupled to oxidation of glucose
##D     # list the available NAD species
##D     info("NAD ")
##D     T <- seq(0,120,10)
##D     # oxidation of glucose (C6H12O6)
##D     basis(c("glucose","H2O","NH3","CO2","H+"),c(-3,0,999,-3,-7))
##D     t <- subcrt(c("NAD(ox)-","NAD(red)-2"),c(-12,12),logact=c(0,0),T=T)
##D     # LH07's diagrams are shown per mole of electron (24 e- per 12 NAD)
##D     A <- t$out$A/24/1000
##D     plot(x=T,y=A,xlim=range(T),ylim=c(1.4,5.4),
##D       xlab=axis.label("T"),ylab=axis.label("A",opt="k"),type="l")
##D     text("NAD(ox)-/NAD(red)-2 = 1",x=median(T),y=median(A))
##D     # different activity ratio
##D     t <- subcrt(c("NAD(ox)-","NAD(red)-2"),c(-12,12),logact=c(1,0),T=T)
##D     A <- t$out$A/24/1000
##D     lines(x=T,y=A)
##D     text("NAD(ox)-/NAD(red)-2 = 10",x=median(T),y=median(A))
##D     # different activity ratio
##D     t <- subcrt(c("NAD(ox)-","NAD(red)-2"),c(-12,12),logact=c(0,1),T=T)
##D     A <- t$out$A/24/1000
##D     lines(x=T,y=t$out$A/24/1000)
##D     text("NAD(ox)-/NAD(red)-2 = 0.1",x=median(T),y=median(A))
##D     # this command prints the reaction on the plot
##D     text(40,4.5,c2s(s2c(describe(t$reaction,
##D       use.name=c(TRUE,TRUE,FALSE,FALSE,FALSE,FALSE)),
##D       sep="=",move.sep=TRUE),sep="\n"))
##D     # label the plot
##D     title(main=paste("Chemical affinity of NAD reduction",
##D      "after LaRowe and Helgeson, 2007",sep="\n"),
##D      sub=describe(thermo$basis,T=NULL))
##D 
##D     ### non-ideality calculations -- activity coefficients of 
##D     ### aqueous species as a function of charge, temperature,
##D     ### and ionic strength -- after Alberty, 2003 
##D     ## p. 16 Table 1.3  apparent pKa of acetic acid with
##D     ## changing ionic strength
##D     subcrt(c("acetic acid","acetate","H+"),c(-1,1,1),
##D       IS=c(0,0.1,0.25),T=25,property="logK")
##D     # note that these *apparent* values of G and logK approach
##D     # their *standard* counterparts as IS goes to zero.
##D     ## p. 95: basis and elemental stoichiometries of species 
##D     ## (a digression here from the nonideality calculations) 
##D     # note coefficient of O2 and NH3 will be zero for these species
##D     basis(c("ATP-4","H+","H2O","HPO4-2","O2","NH3"))
##D     # cf Eq. 5.1-33: (basis composition) 
##D     species(c("ATP-4","H+","H2O","HPO4-2","ADP-3","HATP-3","HADP-2","H2PO4-"))
##D     lb <- length(basis())
##D     # cf Eq. 5.1-32: (elemental composition)
##D     as.matrix(species()[,1:lb]) 
##D     ## p. 273-275: activity coefficient (gamma)
##D     ## as a function of ionic strength and temperature
##D     ## (as of 20080304, these do look quantitatively different 
##D     ## from the plots in Alberty's book.)
##D     iplotfun <- function(T,col,add=TRUE) {
##D       IS <- seq(0,0.25,0.0025)
##D       s <- subcrt(c("H2PO4-","HADP-2","HATP-3","ATP-4"),IS=IS,grid="IS",T=T)
##D       if(!add) thermo.plot.new(xlim=range(IS),ylim=c(0,1),
##D         xlab=axis.label("IS"),ylab="gamma")
##D       for(i in 1:4) lines(IS,10^s$out[[i]]$loggam,col=col)
##D     }
##D     iplotfun(0,"blue",add=FALSE)
##D     iplotfun(25,"black")
##D     iplotfun(40,"red")
##D     title(main=paste("activity coefficient (gamma) of -1,-2,-3,-4",
##D       "charged species at 0, 25, 40 deg C, after Alberty, 2003",
##D       sep="\n"),cex.main=0.95)
##D   
## End(Not run)



