### Name: utilities
### Title: Utility and Miscellaneous Functions
### Aliases: utilities c2s s2c TP.args state.args eos.args GHS
###   can.be.numeric expand.formula protein.length aminoacids nucleicacids
###   thermo.plot.new thermo.postscript axis.label water.lines dPdTtr Ttr
###   ZC label.plot thermo.axis describe basis.comp MP90.cp element
###   mod.obigt which.pmax nonideal change add.protein add.obigt
###   which.balance lsub lsum pprod psum mylapply spearman unitize
###   grep.file read.fasta mtitle examples longex
### Keywords: misc

### ** Examples

  ## Don't show: 
data(thermo)
## End Don't show
  ## string to character
  s2c("hello world")
  s2c("hello world",sep=" ",keep.sep=FALSE)
  s2c("3.141592",sep=c(".","9"))
  s2c("3.141592",sep=c(".","9"),move.sep=TRUE)
  # character to string
  c2s(aminoacids())
  c2s(aminoacids(),sep=".")

  ## Spearman's rho
  spearman(c(1,2,3),c(2,3,4))  # 1
  spearman(c(1,2,3),c(4,3,2))  # -1

  ## argument processing
  eos.args("hkf",c("g","H","S","cP","V","kT","e"))
  ## produces an error because "Q" is not allowed in water.SUPCRT92
  ## Not run: eos.args("hkf",c("G","H","S","Cp","V","kT","E","Q"))
  thermo$opt$water <- "IAPWS"  # needed for p and n in next line
  eos.args("water",c("p","u","cv","psat","rho","n","q","x","y","epsilon"))
  TP.args(c(273.15,373.15))
  TP.args(c(273.15,373.15),"Psat")
  TP.args(c(273.15,373.15),c(100,100,200,200))
  state.args(c("AQ","GAS"))
  state.args(c("a","l","liq"))

  ## converting among Gibbs, enthalpy, entropy
  GHS("H") # entropy of H (element)
  # calculate enthalpy of formation of arsenopyrite 
  GHS("FeAsS",DG=-33843,S=68.5) 
  # return the value of DG calculated from DH and S
  # cf. -56687.71 from subcrt("water")
  GHS("H2O",DH=-68316.76,S=16.7123)  
 
  ## count selected elements in a formula
  t <- makeup("H2O")
  expand.formula(c("H","O"),t)
  expand.formula(c("C","H","S"),t)

  ## count amino acids in a sequence
  aminoacids("GGSGG")
  aminoacids("WhatAmIMadeOf?")

  ## count nucleobases in a sequence
  nucleicacids("ACCGGGTTT")
  # the DNA complement of that sequence
  nucleicacids("ACCGGGTTT",comp="DNA")
  # the RNA complement of the DNA complement
  n <- nucleicacids("ACCGGGTTT",comp="DNA",comp2="RNA")
  # the formula of the RNA complement
  nucleicacids(n,type="RNA")

  ## calculate protein length
  protein.length("LYSC_CHICK")
  # another way to do it
  basis("CHNOS")
  species("LYSC_CHICK")
  protein.length(species()$ispecies)
  # another way to do it
  ip <- protein("LYSC","CHICK")
  protein.length(-ip)

  ## heat capacity as a function of temperature
  ## (Makhatadze & Privalov, 1990) units: J mol-1
  MP90.cp(c(5,25,50,75,100,125),"LYSC_CHICK")

  ## properties of phase transitions
  t <- info("enstatite")
  # (dP/dT) of transitions
  dPdTtr(t)  # first transition
  dPdTtr(t+1) # second transition
  # temperature of transitions (Ttr) as a function of P
  Ttr(t,P=c(1,10,100,1000))
  Ttr(t,P=c(1,10,100,1000))
  
  ## nominal carbon oxidation states
  ZC("CHNOSZ")
  t <- info(info("LYSC_CHICK"))
  ZC(t$formula)

  ## the basis stoichiometry of a made-up species
  # warns because P isn't in our basis
  basis("CHNOS")
  basis.comp("SPONCH")

  ## describing the basis species
  basis("CHNOSe")
  describe(thermo$basis)
  describe(thermo$basis,T=NULL,P=NULL)

  ## mass and entropy of compounds of elements
  element("CH4")
  element(c("CH4","H2O"),"mass")
  element("Z")   # charge
  # same mass, opposite entropy as charge
  element("Z-1") # i.e., electron

  ## scale logarithms of activity
  ## suppose we have two proteins whose lengths
  ## are 100 and 200; what are the logarithms
  ## of activity of the proteins that are equal to
  ## each other and that give a total activity of
  ## residues equal to unity?
  logact <- c(-3,-3)  # could be any two equal numbers
  length <- c(100,200)
  logact.tot <- 0
  loga <- unitize(logact,length,logact.tot)
  # the proteins have equal activity
  stopifnot(identical(loga[1],loga[2]))
  # the sum of activity of the residues is unity
  stopifnot(isTRUE(all.equal(sum(10^loga * length),1)))
  ## now, what if the activity of protein 2 is ten
  ## times that of protein 1?
  logact <- c(-3,-2)
  loga <- unitize(logact,length,logact.tot)
  # the proteins have unequal activity
  stopifnot(isTRUE(all.equal(loga[2]-loga[1],1)))
  # but the activities of residues still add up to one
  stopifnot(isTRUE(all.equal(sum(10^loga * length),1)))

  ## modify/add species
  info(t <- info("alanine","cr"))
  mod.obigt("alanine",state="cr",G=0,H=0,S=0)
  # now the values of G, H, and S are inconsistent
  # with the elemental composition of alanine
  info(t)
  # add a species
  mod.obigt("myname",formula="CHNOSZ",G=0,H=0)
  info(t <- info("myname"))



