### Name: get.protein
### Title: Amino Acid Compositions of Collections of Proteins
### Aliases: get.protein yeastgfp
### Keywords: misc

### ** Examples

  ## Don't show: 
data(thermo)
## End Don't show
  ## basic examples of get.protein
  # amino acid composition of two proteins
  get.protein(c("YML020W","YBR051W"),"SGD")
  # average composition of proteins
  get.protein(c("YML020W","YBR051W"),"SGD",
    abundance=1,pname="PROT1_NEW")
  # 1 of one and 1/2 of the other
  get.protein(c("YML020W","YBR051W"),"SGD",
    abundance=c(1,0.5),average=FALSE,pname="PROT2_NEW")
  # compositions of proteins induced in carbon limitation 
  get.protein("low.C","SGD")

  ## overall composition of proteins exclusively localized 
  ## to cytoplasm of S. cerevisiae with reported expression levels
  y <- yeastgfp("cytoplasm")
  p <- get.protein(y$yORF,"SGD",y$abundance,"cytoplasm")
  # add the proteolog and calculate its properties
  i <- add.protein(p)
  protein(i)

  ## speciation diagram for ER.to.Golgi proteins (COPII coat 
  ## proteins) as a function of logfO2, after Dick, 2009
  y <- yeastgfp("ER.to.Golgi")
  # take out proteins with NA experimental abundance
  ina <- which(is.na(y$abundance))
  y$yORF <- y$yORF[-ina]
  y$abundance <- y$abundance[-ina]
  # get the amino acid compositions of the proteins
  p <- get.protein(y$yORF,"SGD")
  ip <- add.protein(p)
  # use logarithms of activities of proteins such
  # that total activity of residues is unity
  pl <- protein.length(-ip)
  logact <- unitize(rep(1,length(ip)),pl)
  # load the proteins
  basis("CHNOS+")
  a <- affinity(O2=c(-80,-73),iprotein=ip,logact.protein=logact)
  # make a speciation diagram
  diagram(a,ylim=c(-4.9,-2.9))
  # where we are closest to experimental log activity
  logfO2 <- rep(-78,length(ip))
  abline(v=logfO2[1],lty=3)
  # scale experimental abundances such that
  # total activity of residues is unity
  logact.expt <- unitize(log10(y$abundance),pl)
  # plot experimental log activity
  points(logfO2,logact.expt,pch=16)
  text(logfO2+0.5,logact.expt,y$yORF)
  # add title
  title(main=paste("ER.to.Golgi; points - relative abundances",
    "from YeastGFP. Figure after Dick, 2009",sep="\n"))

  ## Chemical activities of model subcellular proteins
  # (one-dimensional speciation diagram as a function of logfO2)
  # After Dick, 2009
  basis("CHNOS+")
  names <- colnames(thermo$yeastgfp)[6:28]
  # calculate amino acid compositions using "get.protein" function 
  for(i in 1:length(names)) {
    y <- yeastgfp(names[i])
    p <- get.protein(y$yORF,"SGD",y$abundance,names[i])
    add.protein(p)
  }
  species(names,"SGD")
  # set unit activity of residues
  pl <- protein.length(thermo$species$name)
  species(NULL,unitize(thermo$species$logact,pl))
  res <- 200
  t <- affinity(O2=c(-82,-65,res))
  mycolor <- topo.colors(6)[1:4]
  mycolor <- rep(mycolor,times=rep(6,4))
  oldpar <- par(bg="black",fg="white")
  logact <- diagram(t,balance="PBB",names=names,ylim=c(-5,-3),legend.x=NULL,
    color=mycolor,lwd=2)$logact
  # so far good, but how about labels on the plot?
  for(i in 1:length(logact)) {
    myloga <- as.numeric(logact[[i]])
    # don't take values that lie above the plot (vacuole in this example)
    myloga[myloga > -3.1] <- -999
    imax <- which.max(myloga)
    adj <- 0.5
    if(imax > 180) adj <- 1
    if(imax < 20) adj <- 0
    text(seq(-82,-65,length.out=res)[imax],logact[[i]][imax],
      labels=names[i],adj=adj)
  }
  title(main=paste("Subcellular proteologs of S. cerevisiae, after Dick, 2009",
    describe(thermo$basis[-5,]),sep="\n"),col.main=par("fg"),cex.main=0.9)
  par(oldpar)

  ### these examples can be run using longex("diagram")
  ## Not run: 
##D     ## Oxygen fugacity - activity of H2O predominance 
##D     ## diagrams for proteologs for 23 YeastGFP localizations
##D     # arranged by decreasing metastability:
##D     # order of this list of locations is based on the 
##D     # (dis)appearance of species on the current set of diagrams
##D     names <- c("vacuole","early.Golgi","ER","lipid.particle",
##D       "cell.periphery","ambiguous","Golgi","mitochondrion",
##D       "bud","actin","cytoplasm","late.Golgi",
##D       "endosome","nucleus","vacuolar.membrane","punctate.composite",
##D       "peroxisome","ER.to.Golgi","nucleolus","spindle.pole",
##D       "nuclear.periphery","bud.neck","microtubule")
##D     nloc <- c(4,5,3,4,4,3)
##D     inames <- 1:length(names)
##D     # define the system
##D     basis("CHNOS+")
##D     # calculate amino acid compositions using "get.protein" function 
##D     for(i in 1:length(names)) {
##D       y <- yeastgfp(names[i])
##D       p <- get.protein(y$yORF,"SGD",y$abundance,names[i])
##D       add.protein(p)
##D     }
##D     species(names,"SGD")
##D     t <- affinity(H2O=c(-5,0,256),O2=c(-80,-66,256))
##D     # setup the plot
##D     layout(matrix(c(1,1,2:7),byrow=TRUE,nrow=4),heights=c(0.7,3,3,3))
##D     par(mar=c(0,0,0,0))
##D     plot.new()
##D     text(0.5,0.5,paste("Subcellular proteologs of S. cerevisiae,",
##D      "after Dick, 2009\n",describe(thermo$basis[-c(2,5),])),cex=1.5)
##D     opar <- par(mar=c(3,4,1,1),xpd=TRUE)
##D     for(i in 1:length(nloc)) {
##D       diagram(t,balance="PBB",names=names[inames],
##D         ispecies=inames,cex.axis=0.75)
##D       label.plot(letters[i])
##D       title(main=paste(length(inames),"locations"))
##D       # take out the stable species
##D       inames <- inames[-(1:nloc[i])]
##D     }
##D     # return to plot defaults
##D     layout(matrix(1))
##D     par(opar)
##D 
##D     ## Compare calculated and experimenal relative abundances
##D     ## of proteins in a subcellular location, after Dick, 2009
##D     # get the amino acid composition of the proteins
##D     loc <- "vacuolar.membrane"
##D     y <- yeastgfp(loc)
##D     ina <- which(is.na(y$abundance))
##D     p <- get.protein(y$yORF[-ina],"SGD")
##D     add.protein(p)
##D     # set up the system
##D     basis("CHNOS+")
##D     # this is the logfO2 value that gives the best fit (see paper)
##D     basis("O2",-74)
##D     is <- species(p$protein,p$organism)
##D     np <- length(is)
##D     pl <- protein.length(species()$name)
##D     # we use unitize so total activity of residues is unity
##D     loga <- rep(0,np)
##D     species(1:np,unitize(loga,pl))
##D     a <- affinity()
##D     d <- diagram(a,do.plot=FALSE)
##D     calc.loga <- as.numeric(d$logact)
##D     expt.loga <- unitize(log10(y$abundance[-ina]),pl)
##D     # which ones are outliers
##D     rmsd <- sqrt(sum((expt.loga-calc.loga)^2)/np)
##D     residuals <- abs(expt.loga - calc.loga)
##D     iout <- which(residuals > rmsd)
##D     pch <- rep(16,length(is))
##D     pch[iout] <- 1
##D     # the colors reflect average oxidation number of carbon
##D     ZC <- ZC(thermo$obigt$formula[species()$ispecies])
##D     col <- rgb(0.15-ZC,0,0.35+ZC,max=0.5)
##D     # there is a color-plotting error on line 567 of the plot.R file 
##D     # of Dick, 2009 that can be reproduced with
##D     #col <- rep(col,length.out=9)
##D     xlim <- ylim <- extendrange(c(calc.loga,expt.loga))
##D     thermo.plot.new(xlim=xlim,ylim=ylim,xlab=expression(list("log"*italic(a),
##D       "calc")),ylab=expression(list("log"*italic(a),"expt")))
##D     points(calc.loga,expt.loga,pch=pch,col=col)
##D     lines(xlim,ylim+rmsd,lty=2)
##D     lines(xlim,ylim-rmsd,lty=2)
##D     title(main=paste("Calculated and experimental relative abundances of\n",
##D       "proteins in ",loc,", after Dick, 2009",sep=""),cex.main=0.95)
##D     
##D 
##D     ### examples for stress response experiments
##D 
##D     ## predominance fields for overall protein 
##D     ## compositions induced by
##D     ## carbon, sulfur and nitrogen limitation
##D     ## (experimental data from Boer et al., 2003)
##D     expt <- c("low.C","low.N","low.S")
##D     for(i in 1:length(expt)) {
##D       p <- get.protein(expt[i],"SGD",abundance=1)
##D       add.protein(p)
##D     }
##D     basis("CHNOS+") 
##D     basis("O2",-75.29)
##D     species(expt,"SGD")
##D     a <- affinity(CO2=c(-5,0),H2S=c(-10,0))
##D     diagram(a,balance="PBB",names=expt,color=NULL)
##D     title(main=paste("Metastabilities of proteins induced by",
##D       "carbon, sulfur and nitrogen limitation",sep="\n"))
##D 
##D     ## predominance fields for overall protein compositions 
##D     ## induced and repressed in an/aerobic carbon limitation
##D     ## (experiments of Tai et al., 2005)
##D     # the activities of glucose, ammonium and sulfate
##D     # are similar to the non-growth-limiting concentrations
##D     # used by Boer et al., 2003
##D     basis(c("glucose","H2O","NH4+","hydrogen","SO4-2","H+"),
##D       c(-1,0,-1.3,999,-1.4,-7))
##D     # the names of the experiments in thermo$stress
##D     expt <- c("Clim.aerobic.down","Clim.aerobic.up",
##D       "Clim.anaerobic.down","Clim.anaerobic.up")
##D     # here we use abundance to indicate that the protein
##D     # compositions should be summed together in equal amounts
##D     for(i in 1:length(expt)) {
##D       p <- get.protein(expt[i],"SGD",abundance=1)
##D       add.protein(p)
##D     }
##D     species(expt,"SGD")
##D     a <- affinity(C6H12O6=c(-35,-20),H2=c(-20,0))
##D     diagram(a,color=NULL,as.residue=TRUE)
##D     title(main=paste("Metastabilities of average protein residues in",
##D       "an/aerobic carbon limitation in yeast",sep="\n"))
##D   
## End(Not run)



