### Name: protein
### Title: Properties of Proteins
### Aliases: protein protein.residue protein.info residue.info
### Keywords: misc

### ** Examples

  ## Don't show: 
data(thermo)
## End Don't show
  ### Interaction with the 'protein'function

  ## Thermodynamic properties of proteins
  # get the composition of a protein
  protein("BPT1_BOVIN")
  # retrieve the rownumber of a protein in thermo$protein
  iprotein <- protein("LYSC","CHICK")
  # calculate properties and parameters of aqueous protein
  protein(iprotein)
  # of crystalline protein
  protein(iprotein,"cr")
  # a call to info() causes the protein properties to
  # be appended to thermo$obigt                         
  info("LYSC_CHICK")
  # thermodynamic properties can be calculated with subcrt()
  subcrt("LYSC_CHICK")                          

  ### Table of properties of some proteins
  basis("CHNOS+")
  species(c("LYSC_CHICK","CYC_BOVIN","MYG_HORSE","RNAS1_BOVIN"))
  protein.info()
  # the following gives the per-residue composition (i.e. formation 
  # reaction cofficients) for the ionized proteins
  residue.info()

  ## Protein Data from Online Sources
  ## Not run: 
##D     ## marked dontrun because it requires internet
##D     # this asks to search SWISS-Prot
##D     info("PRND_HUMAN")
##D     # an online search can also be started from the
##D     # "subcrt" function
##D     subcrt("SPRN_HUMAN")
##D   
## End(Not run)  ## end dontrun

  ## Inputting protein compositions
  # make a new protein
  protein("GGSGG","PROTEIN_TEST")
  # a sequence can be pasted into the command line:
  # type this
  protein("
  # then paste the sequence
  # and end the command by typing
  ","PROTEIN_NEW")
  # or use whatever name you want (with an underscore).

  ## Standard molal entropy of a protein reaction
  basis("CHNOS")
  # here we provide the reaction coefficients of the 
  # proteins (per protein backbone); 'subcrt' function calculates 
  # the coefficients of the basis species in the reaction
  t <- subcrt(c("CSG_METTL","CSG_METJA"),c(-1/530,1/530),
    T=seq(0,350,length.out=50))
  thermo.plot.new(xlim=range(t$out$T),ylim=range(t$out$S),
    xlab=axis.label("T"),ylab=axis.label("DS0r"))
  lines(t$out$T,t$out$S)
  # do it at high pressure as well
  t <- subcrt(c("CSG_METTL","CSG_METJA"),c(-1/530,1/530),
    T=seq(0,350,length.out=50),P=3000)
  lines(t$out$T,t$out$S,lty=2)
  # label the plot
  title(main=paste("Standard molal entropy\n",
    "P = Psat (solid), P = 3000 bar (dashed)"))
  t$reaction$coeff <- round(t$reaction$coeff,3)
  d <- describe(t$reaction,
    use.name=c(TRUE,TRUE,FALSE,FALSE,FALSE,FALSE,FALSE))
  text(170,-3,c2s(s2c(d,sep="=",move.sep=TRUE),sep="\n"),cex=0.8)

  ### Metastability calculations

  ### these examples can be run using longex("protein")
  ## Not run: 
##D     ## subcellular homologs of yeast glutaredoxin
##D     ## as a function of logfO2 - logaH2O, after Dick, 2009
##D     basis("CHNOS+")
##D     protein <- c("GLRX1","GLRX2","GLRX3","GLRX4","GLRX5")
##D     loc <- c("(C)","(M)","(N)","(N)","(M)")
##D     species(protein,"YEAST")
##D     t <- affinity(H2O=c(-10,0),O2=c(-85,-60))
##D     diagram(t,names=paste(protein,loc))
##D     title(main=paste("Yeast glutaredoxins (black) and residues (blue)\n",
##D       describe(thermo$basis[-c(2,5),])))
##D     # note the difference when we set as.residue=TRUE to
##D     # plot stability fields for the residue equivalents of the
##D     # proteins instead of the proteins themselves ...
##D     # the residue equivalent for one of the larger proteins appears
##D     diagram(t,names=paste(protein,loc),as.residue=TRUE,
##D       add=TRUE,col="blue")
##D 
##D     ## surface-layer proteins from Methanococcus and others:
##D     ## a speciation diagram for surface layer proteins
##D     ## as a function of oxygen fugacity after Dick, 2008
##D     # make our protein list
##D     organisms <- c("METSC","METJA","METFE","HALJP","METVO",
##D       "METBU","ACEKI","BACST","BACLI","AERSA")
##D     proteins <- c(rep("CSG",6),rep("SLAP",4))
##D     proteins <- paste(proteins,organisms,sep="_")
##D     # set some graphical parameters
##D     lwd <- c(rep(3,6),rep(1,4))
##D     lty <- c(1:6,1:4)
##D     # load the basis species and proteins
##D     basis("CHNOS+")
##D     species(proteins)
##D     # calculate affinities
##D     a <- affinity(O2=c(-100,-65))
##D     # make diagram
##D     d <- diagram(a,ylim=c(-5,-1),legend.x=NULL,lwd=lwd,
##D       ylab=as.expression(quote(log~italic(a[j]))),yline=1.7)
##D     # label diagram
##D     text(-80,-1.9,"METJA")
##D     text(-74.5,-1.9,"METVO")
##D     text(-69,-1.9,"HALJP")
##D     text(-78,-2.85,"METBU",cex=0.8,srt=-22)
##D     text(-79,-3.15,"ACEKI",cex=0.8,srt=-25)
##D     text(-81,-3.3,"METSC",cex=0.8,srt=-25)
##D     text(-87,-3.1,"METFE",cex=0.8,srt=-17)
##D     text(-79,-4.3,"BACST",cex=0.8)
##D     text(-85.5,-4.7,"AERSA",cex=0.8,srt=38)
##D     text(-87,-4.25,"BACLI",cex=0.8,srt=30)
##D     # add water line
##D     abline(v=-83.1,lty=2)
##D     title(main=paste("Surface-layer proteins",
##D       "After Dick, 2008",sep="\n"))
##D 
##D     ## relative metastabilities of bovine proteins, 
##D     ## as a function of temperature along a glutathione redox buffer
##D     mod.buffer("GSH-GSSG",c("GSH","GSSG"),logact=c(-3,-7))   
##D     basis(c("CO2","H2O","NH4+","SO4-2","H2","H+"),
##D       c(-1,0,-4,-4,"GSH-GSSG",-7)) 
##D     basis("CO2","gas")
##D     species(c("CYC","RNAS1","BPT1","ALBU","INS","PRIO"),"BOVIN")
##D     a <- affinity(T=c(0,200))
##D     diagram(a,as.residue=TRUE,ylim=c(-2,0.5))
##D     title(main="Bovine proteins")
##D   
## End(Not run)



